/******************************************************************************
*
* project name:    Kalaha
* file name:       Ai21.c
* initial date:    15.09.2004
* author:          mhubi@freesurf.ch
* description:     artifical intelligence
* 
* last changes:		 19.01.2004 14:20
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
**************************************************************************** */


//include the main-header-file
#include "main-h21.h"

//extern-variables used in this c-file
extern unsigned int pot[8][15];
extern GAME game;
extern PLAYER player[2];
extern AI ai;


//main-routine of the ai
void DynamicAi(void) {
	
	//SET THE START-VALUES;
	int optval = -73;

	ai.player = game.cur_pl;
	ai.ennemi = next_player[game.cur_pl];
	ai.end_node = 0;
	ai.max_depth = player[game.cur_pl].ai_strength;
	ai.end_move = 0;
	ai.evaluate = 0;
	
	#ifndef GRAPHICAL_VERSION
	//delete the middle part of the screen (only in the non-graphical-version)
	//own function
	MsgDelMiddle();
	
	//print the depth on the middle part of the screen
	//own function
	MsgDrawAidepth();
	
	#endif
	
	//CALL THE MINIMAX-FUNCTION;
	//own function
	optval = MiniMax(1, ai.player, -100, 100);
	
	#ifndef GRAPHICAL_VERSION
	//print all the end-ai-values (only in the non-graphical-version)
	//own function
	MsgDrawAiEnd(optval);
	
	#endif
	
	//RETURN THE CHOSEN MOVE TO THE GAME-MAIN-ROUTINE;
	game.cur_pos = ai.end_move;
	
}



//the recursive minimax function
int MiniMax(int depth, unsigned int player, int alpha, int beta) {
	
	//variables
	unsigned int loop1, loop2, optmove, begin, equal, planestatus;
	int optval, value;
	
	//SET THE START-VALUES FOR THIS DEPTH;
	planestatus = NORMAL;
	ai.next_pl = next_player[player];
	begin = 1 + player*7;
	optmove = 0;
	equal = 0;
	
	//dependent of the current player
	if (player == ai.ennemi) {
		optval = 73;
	}
	else {
		optval = -73;
	}
	
	//FOR (LOOP THROUGH THE SIX POSSIBLE POTS)
	for (loop1 = begin + 5; loop1 >= begin && planestatus != CUTOFF; loop1--) {
		
		planestatus = NORMAL;
		
		#ifndef GRAPHICAL_VERSION
		//show the progress (only in the non-graphical-version)
		if (ai.end_node%1000 == 0) {
			
			//each thousand end-node update the screen
			//own function
			MsgDrawAiNodes();
			
		}
		#endif
		
		//fastquit-possibility
		//tigcclib: kbd.h
		if (_keytest(RR_F2)) {
			
			//set the status value
			game.status = FASTQUIT;
			
			//call the endblock of this program
			LeadOut();
			
		}
		
		//IF (THERE'S A STONE IN THIS POT IN THE LAST DEPTH)
		if (pot[depth - 1][loop1] != 0) {
			
			//COPY THE FIELD FROM THE LAST DEPTH;
			for (loop2 = 1; loop2 <= 14; loop2++) {
				pot[depth][loop2] = pot[depth-1][loop2];
			}
			
			//EXECUTE THE MOVE FROM THE CURRENT POT;
			//own functions
			if (game.rule == 1) {
				AiExecMove1(loop1, depth, player);
			}
			else {
				AiExecMove2(loop1, depth, player);
			}
			
			//init each loop the value-variable before evaluating
			if (player == ai.ennemi) {
				value = 73;
			}
			else {
				value = -73;
			}
			
			//set the planestatus for an endnode
			if (depth == ai.max_depth) {
				planestatus = ENDNODE;
			}
			
			//IF (AT THIS POINT THE GAME WOULD END)
			//SET -72, 0 OR 72;
			//own function
			value = AiCheckWinner(depth, value, &planestatus);
						
			//ELSE IF (IT'S AN ENDNODE)
			//CALL THE SITUATION-ANALYZER;
			if (planestatus == ENDNODE) {
				
				//increase variables
				ai.end_node++;
				ai.evaluate++;
				
				//evaluation
				value = pot[depth][(ai.player + 1)*7] - pot[depth][(ai.ennemi + 1)*7];
				
			}
			
			//ELSE
			//CALL THIS FUNCTION FOR THE NEXT DEPTH
			//(PARAMETER ARE THE NEXT PLAYER, THE NEXT DEPTH, THE VALUE OF ALPHA AND BETA);
			else if (planestatus != NEXTNODE) {
				
				//call the recursive function for the next depth
				//own function
				value = MiniMax(depth + 1, ai.next_pl, alpha, beta);
				
				//refresh the global variable for this depth
				ai.next_pl = next_player[player];
				
			}
			
			//IF (THE AI-PLAYER HAS EXECUTED THE MOVE)
			//IF (THE VALUE IS A NEW MAXIMUM)
			//SAVE THE VALUE AND THE MOVE-NUMBER;
			if (player == ai.player) {
				
				//save it if it is
				if (value > optval) {
					
					optval = value;
					optmove = loop1;
					
				}
				
				//ELSE IF (THE VALUE IS EQUAL TO THE MAXIMUM)
				//CHOOSE RANDOMLY BETWEEN THE TWO MOVES;
				else if (value == optval) {
					
					//if equal choose it randomly
					equal++;
					
					//tigcclib: stdlib.h
					if (random(equal + 1) == 0) {
						optmove = loop1;
					}
					
				}
				
				//IF (THE MAXIMUM IS BIGGER THAN THE BETA-VALUE)
				//BETA-CUTOFF;
				if (optval > beta) {
					planestatus = CUTOFF;
				}
				
				//ELSE IF (THE MAXIMUM IS BIGGER THAN THE ALPHA-VALUE)
				//UPDATE THE ALPHA-VALUE;
				else if (optval > alpha) {
					alpha = optval;
				}
				
				
			}
			
			//ELSE
			//IF (THE VALUE IS A NEW MINIMUM)
			//SAVE THE VALUE AND THE MOVE-NUMBER;
			else if (player == ai.ennemi) {
				
				//if it is save it
				if (value < optval) {
					
					optval = value;
					optmove = loop1;
					
				}
				
				//ELSE IF (THE VALUE IS EQUAL TO THE MINIMUM)
				//CHOOSE RANDOMLY BETWEEN THE TWO MOVES;
				else if (value == optval) {
					
					//if it's equal
					equal++;
					
					//choose it randomly
					//tigcclib: stdlib.h
					if (random(equal + 1) == 0) {
						optmove = loop1;
					}
					
				}
				
				//IF (THE MINIMUM IS LOWER THAN THE ALPHA-VALUE)
				//ALPHA-CUTOFF;
				if (optval < alpha) {
					planestatus = CUTOFF;
				}
				
				//ELSE IF (THE MINIMUM IS LOWER THAN THE BETA-VALUE)
				//UPDATE THE BETA-VALUE;
				else if (optval < beta) {
					beta = optval;
				}
				
			}
		
		}
			
	}
	
	//RETURN THE MAXIMUM/MINIMUM (IN DEPTH 1 THE MOVE-NUMBER TOO);
	if (depth == 1) {
		ai.end_move = optmove;
	}
	
	return optval;
	
}


//distributes the stones of the chosen pot among the next pots, rules #1, virtual field
void AiExecMove1(unsigned int position, unsigned int field, unsigned int player) {
	
	unsigned int potnum, stones;
	
	//everything happens in a virtual field (depending on the depth)
	
	//clear the chosen pot (number of stones in a variable)
	stones = pot[field][position];
	pot[field][position] = 0;
	
	
	//on each following pot a stone while there's one left
	for (potnum = position + 1; stones > 0; stones--, potnum++) {
		
		//set back so there's an array-circle
		if (potnum > 14) {
			potnum = 1;
		}
		
		//increase the content of a pot
		pot[field][potnum]++;
		
		
	}

	//checking the special-rules
	//own function
	AiCheckRules1(potnum - 1, field, player);
	
}


//distributes the stones of the chosen pot among the next pots, rules #2, virtual field
void AiExecMove2(unsigned int position, unsigned int field, unsigned int player) {
	
	unsigned int potnum, stones;
	
	//everything happens in a virtual field (depending on the depth)
	
	//clear the chosen pot (number of stones in a variable)
	stones = pot[field][position];
	pot[field][position] = 0;
	
	
	//on each following pot a stone while there's one left
	for (potnum = position + 1; stones > 0; stones--, potnum++) {
		
		//set back so there's an array-circle
		if (potnum > 14) {
			potnum = 1;
		}
		else if (potnum == 14 && position < 7) {
			potnum = 1;
		}
		else if (potnum == 7 && position > 7) {
			potnum = 8;
		}
		
		//increase the content of a pot
		pot[field][potnum]++;
		
		
	}

	//checking the special-rules
	//own function
	AiCheckRules2(potnum - 1, field, player);
	
}


//checks if in this case the rules must be respected, rules #1, virtual field
void AiCheckRules1(unsigned int potnum, unsigned int field, unsigned int player) {
	
	//if the last stones lands in the own big pot
	if (potnum/(player + 1) == 7) {
		
			//increases the player by two so he's also the next player
			ai.next_pl = next_player[player + 2];
	
	}
	
	//if the first rule doesn't have to be respected check the second rule
	//check if the last stone doesn't end in a big pot, ends on the own side and there's only one stone in
	//this pot
	
	else if (potnum%7 != 0 && potnum > 7*player && potnum < (player + 1)*7 && pot[field][potnum] == 1) {
		
		//the big pot gets the stones of the opposite pot
		pot[field][(player + 1)*7] += pot[field][14 - potnum];
		pot[field][14 - potnum] = 0;
		
	}
	
	
}


//checks if in this case the rules must be respected, rules #2, virtual field
void AiCheckRules2(unsigned int potnum, unsigned int field, unsigned int player) {
	
	//if the last stones lands in the own big pot
	if (potnum/(player + 1) == 7) {
		
			//increases the player by two so he's also the next player
			ai.next_pl = next_player[player + 2];
	
	}
	
	//if the first rule doesn't have to be respected check the second rule
	//check if the last stone doesn't end in a big pot, ends on the own side and there's only one stone in
	//this pot
	
	else if (potnum%7 != 0 && potnum > player*7 && potnum < (player + 1)*7 && pot[field][potnum] == 1 && pot[field][14-potnum] != 0) {
		
		//the big pot gets the stones of the opposite pot
		pot[field][(player + 1)*7] += pot[field][14 - potnum] + 1;
		pot[field][14 - potnum] = 0;
		pot[field][potnum] = 0;
		
	}
	
	
}


//check if the game would end, virtual field
int AiCheckWinner(unsigned int field, int value, unsigned int *planestatus) {
	
	unsigned int loop1, sum;
	
	//if one condition is true you don't have to check other cases --> else
	//check if there are more than 36 stones in a big pot and than set value for the end of the search-tree
	if (pot[field][(ai.ennemi + 1)*7] > game.stones*6) {
		
		value = -72;
		*planestatus = NEXTNODE;
		ai.end_node++;
		
	}
	
	else if (pot[field][(ai.player + 1)*7] > game.stones*6) {
		
		value = 72;
		*planestatus = NEXTNODE;
		ai.end_node++;
		
	}
	
	else if (pot[field][7] == game.stones*6 && pot[field][14] == game.stones*6) {
		
		value = 0;
		*planestatus = NEXTNODE;
		ai.end_node++;
		
	}

	else {
		
		//look if the next player doesn't have any stones --> the other one is the winner
		sum = 0;
		
		//sum up all the stones in the little pots of the next player, but exits if there is still a stone
		for (loop1 = 1; loop1 <= 6 && sum == 0; loop1++) {
			sum += pot[field][loop1 + ai.next_pl*7];
		}
		
		//if there aren't any stones on the next-player's side, set the winner-value and the status
		if (sum == 0) {
		
			if (ai.next_pl == ai.player) {
				value = -72;
			}
			
			else if (ai.next_pl == ai.ennemi) {
				value = 72;
			}
			
			*planestatus = NEXTNODE;
			ai.end_node++;
			
		}
		
	}
	
	//return the value which maybe is changed 
	return value;
	
}


//=============================================================================
// file history
//=============================================================================
//
//	31.7.2004		added randomai()
//	 2.8.2004 	added staticai()
//  13.8.2004		added the license
//	15.9.2004		deleted randomai() and staticai() in this version; added dynamicai()
//							minimax(), aiexecmove(), aicheckrules(), aicheckwinner()
//	28.9.2004-
//	 2.10.2004	added the alpha-beta-things (until 6.10.04 without major-bugs)
//	11.10.2004	changed aicheckwinner (-->variable number of stones)
//	13.10.2004	bug fixes in dynamicai
//	 4.11.2004	added ifndef's
//  19.01.2004	fixed bug in aicheckrules2
