/******************************************************************************
*
* project name:    Kalaha
* file name:       Main21.c
* initial date:    15.09.2004
* author:          mhubi@freesurf.ch
* description:     main-file
* 
* last changes:		 13.10.2004 12:00
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//include the main-header-file
#include "main-h21.h"

//extern-variables used in this c-file
extern INT_HANDLER save_int_1;
extern INT_HANDLER save_int_5;
extern GAME game;
extern PLAYER player[2];
extern int m_choice;
extern LCD_BUFFER homescreen;


//endblock of the game (idea from Excitebike 68k by Travis Fischer)
void LeadOut(void) {
	
	//own function
	Wait();
	
	//disable grayscale
	//tigcclib: gray.h
	GrayOff();
	
	//restore the old interrupt-configuration
	//tigcclib: intr.h
	SetIntVec(AUTO_INT_1, save_int_1);
	SetIntVec(AUTO_INT_5, save_int_5);
	
	//restore the home-screen
	//tigcclib: graph.h
	LCD_restore(homescreen);
	
	if (game.status > FASTQUIT) {
		
		//display a game-message in the status line
		//tigcclib: statline.h
		ST_helpMsg("                             ~~ Kalaha v.2.1    by hubisoft ~~");
		
	}
	else if (game.status == ERROR) {
		
		//display that it was an error in a menu, which "crashed" the program
		//tigcclib: statline.h
		ST_helpMsg("MENUERROR --> ABNORMAL PROGRAM TERMINATION");
		
	}
	
	//exits the game
	//tigcclib: stdlib.h
	exit(0);
	
}


// Main Function (where all the fun starts)
void _main(void) {
	
	//set menu-value
	m_choice = -1;
	
	//save the interrupt-configuration
	//tigcclib: intr.h	
	save_int_1 = GetIntVec(AUTO_INT_1);
	save_int_5 = GetIntVec(AUTO_INT_5);
	
	//save the background
	//(i'm not using SAVE-SCREEN, because i want to display a status-line-message at the end)
	//tigcclib: graph.h
	LCD_save(homescreen);
	
	//set the interrupts to "nothing" (disabled for grayscale and rowread)
	//tigcclib: graph.h
	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
	SetIntVec(AUTO_INT_5, DUMMY_HANDLER);
	
	//seed the random-number-generator
	//tigcclib: stdlib.h
	randomize();
	
	//enable grayscale
	//tigcclib: gray.h
	GrayOn();
	
	//delete the screen
	//own function
	DelScreen();
	
	//init game-variables
	//own function
	InitGame();
	
	//always return to this menu if a load-function failed
	while (m_choice < 0) {
		
		//call the first menu and call the chosen function
		//own function
		MenuMainRoutine(M_1STSCREEN);
		(*menu_func[menu[M_1STSCREEN].str_start + m_choice]) ();
		
	}
	
	//do this not if a game has been loaded
	if (m_choice != 2) {
		
		//reset this variable
		m_choice = 0;
		
		//always return to this menu, if the user didn't choose PLAY
		while (m_choice != 1) {
			
			//call this menu
			//own function
			MenuMainRoutine(M_PLAY);
	 		
	 		//execute the chosen function (if it's not PLAY)
	 		if (m_choice != 1) {
				(*menu_func[menu[M_PLAY].str_start + m_choice]) ();
			}
		
		}
		
		//init the pots
		//own function
		InitPots();
	
	}
	
	//draw the field
	//own function
	DrawEntireField();
	
	//clear the arrow-position
	//own function
	DelArrow();
	
	//own function
	Wait();
	
	//enter into game-loop
	//own function
	GameLoop();
	
	
	//if there's a "positive" result (game ended normally) show the result
	if (game.status > 0) {
		
		//draw the game-end-message
		//own functions
		MenuMainRoutine(M_P0WON  + game.status - 1);
		
		//if there are two different player-types
		if (player[0].type != player[1].type) {
			
			//calculate score and then update the hiscore
			//own function
			CalcScore();
			
		}
		
	}
	
	//call the endblock of this program
	//own function
	LeadOut();
	
}


//=============================================================================
// file history
//=============================================================================
//
//	8.7.2004	start of the multiple-file-project; contains the functions-call and
//						the structure of a game-loop-file (only temporary main-file)
//	9.7.2004	integration of the new functions and the new routines; deleted 
//						#includes
//	12.7.2004	added the whole while() with the whole game-cycle; first version with
//						two players and the extra rules --> game-structure and player-structure
//						--> many changes are necessary (variable-names...)
//	24.7.2004 call drawresults at the end
//	31.7.2004 added the support the calc-player-type (new block), paging of the 
//						arrow-moving
//						the execmove-call isn't separate for each player anymore --> status
//	 2.8.2004 changed the call of the ai
//	12.8.2004 added player inscription
//  13.8.2004	added the license
//	19.8.2004	corrected a bug in the player-inscription
//	31.8.2004 added the background-function-call
//	15.9.2004	added the new ai-call
//	16.9.2004	added the menu-call for choosing the level; save/restore background
//						"myself" for being able to display a statusline-message at the end
//10-13.10.04	added/changed: call for various menus, create game-loop (no longer in main-function),
//						hiscore-calls, bug-fixes
//	2.11.2004 leadout()
